@@ -28,6 +28,7 @@ my $is_windows = ($^O =~ /MSWin32/i) ? 1 : 0;
 my $is_linux = ($^O =~ /linux/i) ? 1 : 0;
 my $is_osx = ($^O =~ /darwin/i) ? 1 : 0;
 my $is_freebsd = ($^O =~ /freebsd/i) ? 1 : 0;
+my $is_openbsd = ($^O =~ /openbsd/i) ? 1 : 0;
 my $is_gkfreebsd = ($^O =~ /gnukfreebsd/i) ? 1 : 0;
 my $is_netbsd = ($^O =~ /netbsd/i) ? 1 : 0;
 my $is_bsd = ($^O =~ /bsd/i) ? 1 : 0;
@@ -186,7 +187,7 @@ if (!$is_windows)
 	$def .= ' -DHAVE_SIGNAL_H -DHAVE_UNISTD_H -DHAVE_DEV_URANDOM';
 }
 
-if ($is_linux || $is_freebsd)
+if ($is_linux)
 {
 	$def .= ' -DHAVE_ENDIAN_H';
 }
@@ -216,7 +217,12 @@ if ($is_bsd || $is_osx)
 
 if ($is_bsd || $is_osx || $is_linux)
 {
-	$def .= ' -DHAVE_GETRANDOM -DHAVE_GETPAGESIZE -DHAVE_SYSCONF';
+	$def .= ' -DHAVE_GETPAGESIZE -DHAVE_SYSCONF';
+}
+
+if ($is_osx || $is_linux)
+{
+	$def .= ' -DHAVE_GETRANDOM';
 }
 
 if ($is_msvc)
@@ -330,11 +336,22 @@ my @fido2srcs = (qw/
 /);
 
 push @fido2srcs, 'hid_linux.c' if ($is_linux);
+push @fido2srcs, 'hid_openbsd.c' if ($is_openbsd);
 push @fido2srcs, 'hid_osx.c' if ($is_osx);
 push @fido2srcs, 'hid_win.c' if ($is_windows);
 
+my @hidapisrcs;
+if ($is_bsd && !$is_openbsd && !$is_windows)
+{
+	push @fido2srcs, 'hid_hidapi.c';
+	push @hidapisrcs, 'deps/hidapi/libusb/hid.c';
+
+	$def .= ' -DUSE_HIDAPI';
+	$lib .= ' -lusb -liconv';
+}
+
 my @srcs = ((map { "deps/libfido2/openbsd-compat/$_" } @fido2compat), (map { "deps/libfido2/src/$_" } @fido2srcs));
-my @objs = map { substr ($_, 0, -1) . 'o' } (@cborsrcs, @srcs);
+my @objs = map { substr ($_, 0, -1) . 'o' } (@cborsrcs, @srcs, @hidapisrcs);
 
 sub MY::c_o {
 	my $out_switch = '-o ';
@@ -519,7 +536,7 @@ TEMPLATE
 override _build_WriteMakefile_args => sub {
 	return +{
 		%{ super() },
-		INC	    => ' -Ideps/config -Ideps/libcbor/src -Ideps/libfido2 -Ideps/libfido2/src',
+		INC	    => ' -Ideps/config -Ideps/libcbor/src -Ideps/libfido2 -Ideps/libfido2/src -Ideps/hidapi -Ideps/hidapi/hidapi',
 		OBJECT	=> '$(O_FILES)',
 	}
 };
